const AsteriskConn = require('./AsteriskConn');
const Fila = require('xcontact-lib/models/Fila');
const Agente = require('xcontact-lib/models/Agente');
const AgenteAmi = require('./models/AgenteAmi');
const AgenteFilas = require('xcontact-lib/models/AgenteFilas');
const LigacoesAmi = require('./models/LigacoesAmi');
const log = require('xcontact-lib/util/log');
const EventEmitter = require('events');
class MyEmitter extends EventEmitter { }
const emitter = new MyEmitter();
const moment = require('moment')
const _ = require('underscore');
const { stat } = require('fs');

module.exports = class Ami {
    constructor() {
        this.filas = [];
    }
    on (event, fn) {
        return emitter.addListener(event, fn);
    }
    /**
     * Limpa eventos do Asterisk
     * @param {Object} response 
     */
    limpa (response) {
        const { lines, EOL, variables, privilege, event, ...ev } = response;
        return ev;
    }
    resetFila (fila) {
        fila.ligacoes_atendidas = [];
        fila.ligacoes_ativas = [];
        fila.ligacoes_abandonadas = [];
        fila.ligacoes_abandonadas_efetivas = [];
        fila.ligacoes_aguardando = [];
        fila.ligacoes_efetuadas = [];
        fila.ligacoes_transbordadas = [];

        fila.total_transbordo = 0;
        fila.total_espera = 0;
        fila.esperas_maior = 0;
        fila.esperas_menor = 0;
        fila.abandonos_maior = 0;
        fila.abandonos_menor = 0;
        fila.total_falando = 0;
        fila.total_abandono = 0;
        fila.max_espera = 0;
        fila.max_falando = 0;
        fila.max_abandono = 0;
        fila.ligacoes_efetuadas_totais = 0;

        // Suporte ao omni
        fila.MsgEnviadas = 0
        fila.MsgRecebidas = 0
        fila.TotalAtendimentosEntrantes = 0
        fila.TotalAtendimentosSaintes = 0
        fila.TotalAtendimentosEncerrados = 0
        fila.tempo_total_resposta = 0
        fila.total_mensagens_resposta = 0
        fila.tempo_primeira_resposta = 0
        fila.total_primeira_resposta = 0
        fila.tmr = 0
        fila.tmpr = 0
        fila.tmaxr = 0
        fila.tmaxpr = 0
        fila.AtendimentosAguardando = []
        fila.clientes_atendidos = []
        fila.AtendimentosEmAndamento = []

        if (!fila.agentes) fila.agentes = [];
    }
    reset () {
        this.filas.forEach(fila => {
            this.resetFila(fila);
        });
    }
    _calcFilaStatus (fila) {
        return {
            nome: fila.name,
            agentes: fila.agentes.length,
            aguardando: fila.ligacoes_aguardando.length,
            atendidas: fila.ligacoes_atendidas.length,
            efetuadas: fila.ligacoes_efetuadas.length,
            abandonadas: fila.ligacoes_abandonadas.length,
            transbordadas: fila.ligacoes_transbordadas.length,
            transbordos: fila.total_transbordo,
            tme: Math.round(fila.total_espera / fila.ligacoes_atendidas.length) || 0,
            tma: Math.round(fila.total_falando / fila.ligacoes_atendidas.length) || 0,
            tmab: Math.round(fila.total_abandono / fila.ligacoes_abandonadas.length) || 0,
            tmr: Math.round(fila.tempo_total_resposta / fila.total_mensagens_resposta) || 0,
            tmpr: Math.round(fila.tempo_primeira_resposta / fila.total_primeira_resposta) || 0,
            tmaxr: fila.tmaxr,
            tmaxpr: fila.tmaxpr,
            max_espera: fila.max_espera,
            max_falando: fila.max_falando,
            max_abandono: fila.max_abandono
        }
    }
    getAllFilasStatus () {
        return this.filas.map(fila => {
            return this._calcFilaStatus(fila);
        })
    }
    getFilaStatus (NomeFila) {
        const fila = this.filas.find(f => f.name == NomeFila);
        if (fila) {
            return this._calcFilaStatus(fila);
        }
    }
    async addAgente (NomeFila, agente) {
        // console.log('------------ addAgente ---------------')
        // console.log(agente)
        if (this.filas.length <= 0) return;
        if (!agente.name) agente.name = agente.membername;
        log.debug(`[addAgente] Adicionando ${agente.name} na fila ${NomeFila}`);

        let fila = this.filas.find(f => f.name === NomeFila);
        if (fila) {
            if (fila.agentes) {
                const agFila = fila.agentes.find(ag => ag.nome === agente.name)
                if (agFila) return
            }
            const AgDb = await Agente.query.require(false).first()
                .where({ fullname: agente.name })
                .fetch();
            if (AgDb) {
                // Busca a fila padrão, primeira fila do agente.
                const AgDbFila = await AgenteFilas.query.require(false).first()
                    .orderBy('id')
                    .where({ id_agente: AgDb.id })
                    .fetch();
                agente.id = AgDb.id;
                agente.modo = AgDb.modo;
                agente.fila_padrao = AgDbFila.fila || undefined;
                agente.max_chat_atende = AgDb.max_chat_atende;
                agente.edita_rota = AgDb.edita_rota;
                agente.visualiza_todos_clientes = AgDb.visualiza_todos_clientes;
            } else return
            const agenteAmi = new AgenteAmi({
                id: agente.id,
                nome: agente.name,
                status: agente.status ? agente.status : 1,
                ramal: agente.stateinterface || AgDb.ramal,
                fila: fila.name,
                fila_padrao: agente.fila_padrao,
                pausa: parseInt(agente.paused),
                pausa_motivo: agente.pausedreason,
                ligacoes_ativas: [],
                atendimento_ativo:[],
                max_chat_atende: agente.max_chat_atende,
                status_date: new Date().getTime(),
                statusDate: moment().unix(),
                ligacoes_atendidas: 0,
                ligacoes_efetuadas: 0,
                ligacoes_efetuadas_totais: 0,
                ligacoes_recebidas: 0,
                ligacoes_naoatendidas: 0,
                ligacoes_naocompletadas: 0,
                atendimentos_recebidos: 0,
                atendimentos_efetuados: 0,
                total_falando: 0,
                total_espera: 0,
                tempo_primeira_resposta: 0,
                total_primeira_resposta: 0,
                tempo_total_resposta: 0,
                total_mensagens_resposta: 0,
                max_espera: 0,
                max_falando: 0,
                tmaxr: 0,
                tmaxpr: 0,
            });
            if (!fila.agentes) fila.agentes = [];
            fila.agentes.push(agenteAmi);

            emitter.emit('AgenteAdicionado', agenteAmi);
            
            return agente;
        }
        else {
            log.warn(); (`[addAgente] Fila ${NomeFila} não encontrada na lista.`);
            return null;
        }
    }
    _delAgente (NomeFila, agente) {
        if (this.filas.length <= 0) return;
        if (!agente.name) agente.name = agente.membername;
        log.debug(`[delAgente] Removendo ${agente.name} da fila ${NomeFila}`);

        let fila = this.filas.find(f => f.name === NomeFila);
        if (fila) {
            if (fila.agentes.length <= 0) return;

            const index = fila.agentes.findIndex(ag => ag.ramal === agente.stateinterface);
            if (index !== -1) {
                const agenteAmi = fila.agentes[index];
                emitter.emit('AgenteRemovido', agenteAmi);
                fila.agentes.splice(index, 1);
            }
        }
        else {
            log.warn(); (`[delAgente] Fila ${NomeFila} não encontrada na lista.`);
        }
    }
    getAgenteByRamal (ramal) {
        // Nunca use forEach, forEach não interromp com return e sempre dá problema em async
        for (let i = 0; i < this.filas.length; i++) {
            if (this.filas[i].agentes.length > 0) {
                const agente = this.filas[i].agentes.find(agente => agente.ramal === ramal);
                if (agente) {
                    return agente;
                }
            }
        }
    }
    getAgentePorNome (nome) {
        // Nunca use forEach, forEach não interromp com return e sempre dá problema em async
        for (let i = 0; i < this.filas.length; i++) {
            if (this.filas[i].agentes.length > 0) {
                const agente = this.filas[i].agentes.find(agente => agente.nome === nome);
                if (agente) {
                    return agente;
                }
            }
        }
    }
    async syncFila (request) {
        try {
            let filadb = await Fila.fetchByName(request.fila);
            let filaarray = this.filas.find(f => f.name == request.fila);
            if (filaarray) {
                filaarray = Object.assign(filaarray, filadb);
                filaarray.nome = filaarray.name;
                filaarray.tipo = filaarray.queue_type;
                filaarray.fila = filaarray.name;
                emitter.emit('FilaAtualizada', filaarray);
                return filaarray;
            }
            else {
                this.resetFila(filadb);
                this.filas.push(filadb);
                filadb.nome = filadb.name;
                filadb.tipo = filadb.queue_type;
                filadb.fila = filadb.name;
                emitter.emit('FilaAtualizada', filadb);
                return filadb;
            }
        } catch (error) {
            console.log(`Erro ao carregar a fila ${request.fila}`);
            console.log(error);
        }
    }
    async RemoverFila (request) {
        const index = this.filas.findIndex(f => f.name == request.fila);
        if (index !== -1) {
            await Fila.query
                .require(false)
                .where({ name: request.fila })
                .delete();

            let filaremovida = this.filas.splice(index, 1);
            filaremovida[0].fila = filaremovida[0].name;

            emitter.emit('FilaRemovida', filaremovida[0]);
            return filaremovida[0];
        }
        else {
            return { error: true, msg: 'Fila não encontrada' };
        }
    }
    async AdicionarFilaAgente (request) {
        try {
            const agf = new AgenteFilas({ id_agente: parseInt(request.id_agente), fila: request.fila });
            let ret = await agf.save();

            emitter.emit('FilaAdicionadaAgente', { id_agente: parseInt(request.id_agente), fila: request.fila });

            return ret;
        } catch (error) {
            console.log(error);
            return null;
        }
    }
    async RemoverFilaAgente (request) {
        try {
            let ret = await AgenteFilas.query
                .require(false)
                .where({ id_agente: parseInt(request.id_agente) })
                .where({ fila: request.fila })
                .delete();

            emitter.emit('FilaRemovidaAgente', { id_agente: parseInt(request.id_agente), fila: request.fila });

            return ret;
        } catch (error) {
            console.log(error);
            return null;
        }
    }
    async start () {
        log.info('[AMI] Start: Serviço iniciado');
        // Primeiro vamos carregar as filas no banco pq o Asterisk as vezes não traz todas as filas.
        try {
            this.filas = await Fila.fetch();
            this.reset();
        } catch (error) {
            this.filas = [];
        }

        this.asterConn = new AsteriskConn();
        const vm = this;
        this.asterConn.on('ConexaoAmiInciada', event => {
            vm.asterConn.getFilas(undefined, async (retorno) => {
                // Aqui só no interesa os Agentes que estão logados para popular na fila
                if (retorno.response == 'Success') {
                    retorno.events.map(evento => {
                        if (evento.event == 'QueueMember') {
                            vm.addAgente(evento.queue, vm.limpa(evento));
                        }
                    })
                    emitter.emit('Iniciado', { error: false, msg: 'iniciado com sucesso' });
                }
            })
        });

        this.asterConn.on('AgenteAdicionado', async event => {
            const tem = await this.addAgente(event.queue, event)
            if (!tem) {
                log.debug(`Fila ${event.queue} não encontrada, tentando syncronizar com o banco`)
                await this.syncFila({ fila: event.queue })
                this.addAgente(event.queue, event)
            }
        });
        this.asterConn.on('AgenteRemovido', event => {
            this._delAgente(event.queue, event);
        });
        this.asterConn.on('AgenteStatus', event => {
            console.log('AgenteStatus', event)
            if (this.filas.length <= 0) return;

            if (!event.name) event.name = event.membername;
            log.debug(`[AgenteStatus] Agente ${event.name} da fila ${event.queue} mudou para status ${event.status}`);

            let fila = this.filas.find(f => f.name === event.queue);
            if (fila) {
                if (fila.agentes.length <= 0) return;

                const agt = fila.agentes.find(ag => ag.nome === event.name);
                if (agt) {
                    agt.status = parseInt(event.status);
                    agt.status_date = new Date().getTime();
                    agt.statusDate = moment().unix();
                    agt.pausa = parseInt(event.paused);
                    agt.pausa_motivo = event.pausedreason;

                    emitter.emit('AgenteStatus', agt);
                }
            }
            else {
                log.warn(); (`[AgenteStatus] Fila ${event.queue} não encontrada na lista.`);
            }
        });

        this.asterConn.on('LigacaoStatus', event => {
            console.log('LigacaoStatus', event);
            const agente = this.getAgenteByRamal(event.channel.split('-')[0]);
            // console.log(agente);
            if (agente) {
                if (!agente.ligacoes_ativas) agente.ligacoes_ativas = [];

                let nova = false;
                let ligacao = agente.ligacoes_ativas.find(lig => lig.callid === event.linkedid);
                if (!ligacao) {
                    nova = true;
                    ligacao = new LigacoesAmi({
                        callid: event.linkedid,
                        inicio: new Date().getTime(),
                        statusDate: moment().unix(),
                        fila: agente.fila_padrao,
                        agente_id: agente.id,
                        agente_nome: agente.nome,
                        agente_ramal: agente.ramal
                    })
                }

                // Ligacao sainte
                if (event.channelstate == '4' || event.exten.length === 0) {
                    ligacao.status = LigacoesAmi.StatusMap.get("Chamando");
                    ligacao.sentido = LigacoesAmi.SentidoMap.get("Sainte");
                    ligacao.numero = event.exten;
                    agente.ligacoes_efetuadas_totais++;
                    let fila = this.filas.find(f => f.name === agente.fila_padrao);
                    if (fila) {
                        fila.ligacoes_efetuadas_totais++
                    }
                    if (ligacao.numero) {
                        fila.clientes_atendidos.push(ligacao.numero)
                    }
                    fila.clientes_atendidos = _.uniq(fila.clientes_atendidos)
                }
                // Ligacao entrante
                if (event.channelstate == '5' && event.exten.length > 0) {
                    ligacao.status = LigacoesAmi.StatusMap.get("Chamando");
                    ligacao.sentido = LigacoesAmi.SentidoMap.get('Entrante');
                    ligacao.numero = event.connectedlinenum;
                    ligacao.nome = event.connectedlinename;
                }
                if (event.channelstate == '6') {
                    ligacao.atendida = new Date().getTime();
                    ligacao.status = LigacoesAmi.StatusMap.get("Ativa");
                    // Contabiliza ligação só se completou
                    if (ligacao.sentido == LigacoesAmi.SentidoMap.get('Entrante')) {
                        agente.ligacoes_recebidas++;
                    }
                    else {
                        agente.ligacoes_efetuadas++;
                        // Registra a ligação sainte da fila
                        let fila = this.filas.find(f => f.name === agente.fila_padrao);
                        if (fila) {
                            fila.ligacoes_efetuadas.push(ligacao);
                        }
                        if (ligacao.status === 2) {
                            fila.ligacoes_ativas.push(ligacao);
                            if (fila.ligacoes_abandonadas_efetivas) {
                                const indexesAbandonadas = fila.ligacoes_abandonadas_efetivas.reduce((acc, call, i) => {
                                    if (call.numero === ligacao.numero) {
                                      acc.push(i)
                                    }
                                    return acc
                                  }, [])
                                  indexesAbandonadas.reverse().forEach((indexAb) => {
                                    fila.ligacoes_abandonadas_efetivas.splice(indexAb, 1)
                                  })
                              } else {
                                return
                              }
                        }
                    }
                }

                if (nova) {
                    agente.ligacoes_ativas.push(ligacao);
                }

                emitter.emit('AgenteLigacao', ligacao);
            }
            else {
                log.debug(`Nenhum agente encontrado para ramal ${event.channel.split('-')[0]}`)
            }
        });

        this.asterConn.on('LigacaoFim', event => {
            console.log('LigacaoFim', event);
            const agente = this.getAgenteByRamal(event.channel.split('-')[0]);
            if (agente) {
                const index = agente.ligacoes_ativas.findIndex(lig => lig.callid === event.linkedid);
                if (index !== -1) {
                    const ligacao = agente.ligacoes_ativas[index];
                    // Contabiliza ligações não atendidas ou não completadas
                    if (event.channelstate != '6') {
                        if (ligacao.sentido == LigacoesAmi.SentidoMap.get('Entrante')) {
                            agente.ligacoes_naoatendidas++;
                            ligacao.status = LigacoesAmi.StatusMap.get('Nao_Atendida');
                        }
                        else {
                            agente.ligacoes_naocompletadas++;
                            ligacao.status = LigacoesAmi.StatusMap.get('Nao_Completada');
                        }
                    }
                    else {
                        ligacao.status = LigacoesAmi.StatusMap.get('Finalizada');
                    }

                    ligacao.fim = new Date().getTime();

                    agente.ligacoes_ativas.splice(index, 1);
                    
                    let fila = this.filas.find(f => f.name === agente.fila_padrao);
                    if (fila) {
                        const indexAtiva = fila.ligacoes_ativas.findIndex(ag => ag.callid == ligacao.callid);
                        if (indexAtiva !== -1) {
                            fila.ligacoes_ativas.splice(indexAtiva, 1);
                        }       
                    }
                    //emitter.emit('AgenteLigacao', agente.ligacoes_ativas[index]);
                    emitter.emit('AgenteLigacao', ligacao);
                }
            }
        });

        this.asterConn.on('AgentePausa', event => {
            console.log('AgentePausa', event);
            if (!event.name) event.name = event.membername;

            let fila = this.filas.find(f => f.name === event.queue);
            if (fila) {
                if (fila.agentes.length <= 0) return;

                const agt = fila.agentes.find(ag => ag.nome === event.name);
                if (agt) {
                    agt.status = parseInt(event.status);
                    agt.status_date = new Date().getTime();
                    agt.statusDate = moment().unix();
                    agt.pausa = parseInt(event.paused);
                    agt.pausa_motivo = event.pausedreason;

                    emitter.emit('AgenteStatus', agt);
                }
            }
            else {
                log.warn(); (`[AgentePausa] Fila ${event.queue} não encontrada na lista.`);
            }
        })

        this.asterConn.on('LigacaoEntrouNaFila', ev => {
            console.log('LigacaoEntrouNaFila', ev)
            const fila = this.filas.find(f => f.name == ev.queue);
            if (fila) {
                const ligacaoExistente = fila.ligacoes_aguardando.find(lig => lig.callid === ev.linkedid)
                if (ligacaoExistente) return
                const ligacao = new LigacoesAmi({
                    callid: ev.linkedid,
                    numero: ev.calleridnum,
                    nome: ev.calleridname,
                    fila: ev.queue,
                    posicao_inicial: parseInt(ev.position),
                    statusDate: moment().unix(),
                    total: parseInt(ev.count),
                    inicio: new Date().getTime()
                })
                fila.ligacoes_aguardando.push(ligacao);

                if (ligacao.numero) {
                    fila.clientes_atendidos.push(ligacao.numero)
                }
                fila.clientes_atendidos = _.uniq(fila.clientes_atendidos)

                emitter.emit('FilaLigacaoAdicionada', ligacao);
            }
            else {
                log.error(`[LigacaoEntrouNaFila] Fila ${ev.queue} não encontrada`)
            }
        })

        this.asterConn.on('LigacaoAbandonouNaFila', ev => {
            console.log('LigacaoAbandonouNaFila', ev)
            const fila = this.filas.find(f => f.name == ev.queue);
            if (fila) {
                const index = fila.ligacoes_aguardando.findIndex(ag => ag.callid == ev.linkedid);
                if (index !== -1) {
                    const ligacao = fila.ligacoes_aguardando[index];
                    fila.ligacoes_aguardando.splice(index, 1);

                    ligacao.fim = new Date().getTime();
                    ligacao.posicao_final = parseInt(ev.position);

                    var diff = Math.abs(ligacao.fim - ligacao.inicio);
                    ligacao.espera = Math.round(diff / 1000);

                    // Abandono
                    if (ev.reason && ev.reason == 'NORMAL') {
                        fila.total_abandono += ligacao.espera;
                        if (ligacao.espera > fila.max_abandono) fila.max_abandono = ligacao.espera;
                        if (ligacao.espera > 60) {
                            fila.abandonos_maior++
                        }
                        else {
                            fila.abandonos_menor++
                        }

                        fila.ligacoes_abandonadas.push(ligacao);

                        const abandonoEfetivo = fila.ligacoes_abandonadas_efetivas.find(
                            (call) => call.numero === ligacao.numero)
                        if (!abandonoEfetivo) {
                            fila.ligacoes_abandonadas_efetivas.push(ligacao)
                          }
                        emitter.emit('FilaLigacaoAbandonou', ligacao);
                    }
                    // Transbordo
                    else {
                        fila.ligacoes_transbordadas.push(ligacao);
                        fila.total_transbordo += ligacao.espera;
                        emitter.emit('FilaLigacaoTransbordou', ligacao);
                    }
                }
                else {
                    log.error(`[LigacaoAbandonouNaFila] Não achei a ligação de callid ${ligacao.callid} na fila ${fila.name}`);
                }
            }
            else {
                log.error(`[LigacaoAbandonouNaFila] Fila ${ev.queue} não encontrada`)
            }
        })

        this.asterConn.on('AgenteAtendeu', ev => {
            console.log('AgenteAtendeu', ev)
            const fila = this.filas.find(f => f.name == ev.queue);
            if (fila) {
                const index = fila.ligacoes_aguardando.findIndex(ag => ag.callid == ev.linkedid);
                if (index !== -1) {
                    const ligacao = fila.ligacoes_aguardando[index];
                    const agente = fila.agentes.find(agente => agente.nome === ev.membername);
                    fila.ligacoes_aguardando.splice(index, 1);
                    ligacao.atendida = new Date().getTime();

                    var diff = Math.abs(ligacao.atendida - ligacao.inicio);
                    ligacao.espera = Math.round(diff / 1000);
                    fila.total_espera += ligacao.espera;
                    agente.total_espera += ligacao.espera;
                    agente.ligacoes_atendidas++;
                    if (ligacao.espera > 60) {
                        fila.esperas_maior++
                    }
                    else {
                        fila.esperas_menor++
                    }
                    fila.ligacoes_ativas.push(ligacao);
                    fila.ligacoes_atendidas.push(ligacao);
                    if (ligacao.espera > fila.max_espera) fila.max_espera = ligacao.espera;
                    if (ligacao.espera > agente.max_espera) agente.max_espera = ligacao.espera;

                    ligacao.agente_nome = ev.membername
                    ligacao.agente_ramal = ev.interface
                    ligacao.ring_agente_segundos = ev.ringtime

                    if (fila.ligacoes_abandonadas_efetivas) {
                        const indexesAbandonadas = fila.ligacoes_abandonadas_efetivas.reduce((acc, call, i) => {
                            if (call.numero === ligacao.numero) {
                              acc.push(i)
                            }
                            return acc
                          }, [])
                          indexesAbandonadas.reverse().forEach((indexAb) => {
                            fila.ligacoes_abandonadas_efetivas.splice(indexAb, 1)
                          })
                    } else {
                        return
                    }

                    emitter.emit('FilaLigacaoAtendida', ligacao);
                }
                else {
                    log.error(`[LigacaoSaiuDaFila] Não achei a ligação de callid ${ev.linkedid} na fila ${fila.name}`);
                }
            }
        })

        this.asterConn.on('AgenteDesligou', ev => {
            console.log('AgenteDesligou', ev)
            const fila = this.filas.find(f => f.name == ev.queue);
            if (fila) {
                const index = fila.ligacoes_atendidas.findIndex(ag => ag.callid == ev.linkedid);
                if (index !== -1) {
                    const indexAtiva = fila.ligacoes_ativas.findIndex(ag => ag.callid == ev.linkedid);
                    const agente = fila.agentes.find(agente => agente.nome === ev.membername);
                    if (indexAtiva !== -1) {
                        fila.ligacoes_ativas.splice(indexAtiva, 1);
                    }
                    const ligacao = fila.ligacoes_atendidas[index];
                    ligacao.motivo_fim = LigacoesAmi.MotivoFimMap.get(ev.reason);
                    ligacao.fim = new Date().getTime();
                    ligacao.falando = parseInt(ev.talktime);

                    fila.total_falando += ligacao.falando;
                    agente.total_falando += ligacao.falando;

                    ligacao.agente_nome = ev.membername
                    ligacao.agente_ramal = ev.interface


                    if (ligacao.falando > fila.max_falando) fila.max_falando = ligacao.falando;
                    if (ligacao.falando > agente.max_falando) agente.max_falando = ligacao.falando;
                    emitter.emit('FilaLigacaoTerminou', ligacao);
                }
            }
        })
    }
    /**
     * Retorna todas as filas
     * @returns Array
     */
    getFilas () {
        return this.filas.map(fila => {
            let copy = Object.assign({}, fila);
            copy.nome = fila.name;
            copy.tipo = fila.queue_type;
            
            copy.atendidas = fila.ligacoes_atendidas.length;
            copy.efetuadas = fila.ligacoes_efetuadas.length;
            copy.abandonadas = fila.ligacoes_abandonadas.length;
            copy.transbordadas = fila.ligacoes_transbordadas.length;
            
            copy.tme = Math.round(fila.total_espera / fila.ligacoes_atendidas.length) || 0;
            copy.tma = Math.round(fila.total_falando / fila.ligacoes_atendidas.length) || 0;
            copy.tmab = Math.round(fila.total_abandono / fila.ligacoes_abandonadas.length) || 0;
            copy.tmr = Math.round(fila.tempo_total_resposta / fila.total_mensagens_resposta) || 0;
            copy.tmpr = Math.round(fila.tempo_primeira_resposta / fila.total_primeira_resposta) || 0;
            
            copy.ligacoes_aguardando.forEach(lig => {
                const diff = moment().diff(lig.inicio, 'miliseconds');
                lig.time = diff;
            });
            
            copy.AtendimentosAguardando.forEach(at => {
                const diff = moment().diff(at.data, 'miliseconds');
                at.time = diff;
            });
            
            copy.agentes.forEach(ag => {
                const diff = moment().diff(ag.status_date, 'miliseconds');
                ag.time = diff;
            });
            delete copy.ligacoes_efetuadas;
            delete copy.ligacoes_transbordadas;
            return copy;
        })
    }
}